@@ -59,53 +59,14 @@ static GOptionEntry options[] = {
   { NULL }
 };
 
-static gboolean
-has_bind_mount_for_path (FlatpakBwrap *bwrap, const char *dest_path)
-{
-  guint i;
-
-  for (i = 0; i < bwrap->argv->len; i++)
-    {
-      const char *arg = g_ptr_array_index (bwrap->argv, i);
-
-      if (g_strcmp0 (arg, "--bind") == 0 ||
-          g_strcmp0 (arg, "--bind-try") == 0 ||
-          g_strcmp0 (arg, "--ro-bind") == 0 ||
-          g_strcmp0 (arg, "--ro-bind-try") == 0 ||
-          g_strcmp0 (arg, "--bind-data") == 0 ||
-          g_strcmp0 (arg, "--ro-bind-data") == 0)
-        {
-          /* For all bind mount types, the destination path is at index i+2.
-           *
-           *   --bind/--ro-bind/--bind-try/--ro-bind-try: type, src, dest
-           *   --bind-data/--ro-bind-data: type, fd_string, dest
-           */
-          if (i + 2 < bwrap->argv->len)
-            {
-              const char *dest = g_ptr_array_index (bwrap->argv, i + 2);
-              if (dest != NULL && g_strcmp0 (dest, dest_path) == 0)
-                return TRUE;
-            }
-        }
-    }
-
-  return FALSE;
-}
-
 static void
 add_empty_font_dirs_xml (FlatpakBwrap *bwrap)
 {
-  const char *font_dirs_path = "/run/host/font-dirs.xml";
-
-  /* Check if a bind mount already exists for this path */
-  if (has_bind_mount_for_path (bwrap, font_dirs_path))
-    return;
-
   g_autoptr(GString) xml_snippet = g_string_new ("<?xml version=\"1.0\"?>\n"
                                                  "<!DOCTYPE fontconfig SYSTEM \"urn:fontconfig:fonts.dtd\">\n"
                                                  "<fontconfig></fontconfig>\n");
 
-  if (!flatpak_bwrap_add_args_data (bwrap, "font-dirs.xml", xml_snippet->str, xml_snippet->len, font_dirs_path, NULL))
+  if (!flatpak_bwrap_add_args_data (bwrap, "font-dirs.xml", xml_snippet->str, xml_snippet->len, "/run/host/font-dirs.xml", NULL))
     g_warning ("Unable to add fontconfig data snippet");
 }
 
@@ -254,6 +215,10 @@ flatpak_builtin_build (int argc, char **argv, GCancellable *cancellable, GError
   char pid_str[64];
   g_autofree char *pid_path = NULL;
   g_autoptr(GFile) app_id_dir = NULL;
+  FlatpakContextShares shares;
+  FlatpakContextDevices devices;
+  FlatpakContextSockets sockets;
+  FlatpakContextFeatures features;
 
   context = g_option_context_new (_("DIRECTORY [COMMAND [ARGUMENT…]] - Build in directory"));
   g_option_context_set_translation_domain (context, GETTEXT_PACKAGE);
@@ -475,9 +440,13 @@ flatpak_builtin_build (int argc, char **argv, GCancellable *cancellable, GError
   if (app_context == NULL)
     return FALSE;
 
-  flatpak_context_allow_host_fs (app_context);
   flatpak_context_merge (app_context, arg_context);
 
+  shares = flatpak_run_compute_allowed_shares (app_context);
+  devices = flatpak_run_compute_allowed_devices (app_context);
+  sockets = flatpak_run_compute_allowed_sockets (app_context);
+  features = flatpak_run_compute_allowed_features (app_context);
+
   minimal_envp = flatpak_run_get_minimal_env (TRUE, FALSE);
   bwrap = flatpak_bwrap_new (minimal_envp);
   flatpak_bwrap_add_args (bwrap, flatpak_get_bwrap (), NULL);
@@ -490,7 +459,7 @@ flatpak_builtin_build (int argc, char **argv, GCancellable *cancellable, GError
   if (custom_usr)
     run_flags |= FLATPAK_RUN_FLAG_WRITABLE_ETC;
 
-  run_flags |= flatpak_context_get_run_flags (app_context);
+  run_flags |= flatpak_context_features_to_run_flags (features);
 
   /* Unless manually specified, we disable dbus proxy */
   if (!flatpak_context_get_needs_session_bus_proxy (arg_context))
@@ -597,6 +566,7 @@ flatpak_builtin_build (int argc, char **argv, GCancellable *cancellable, GError
                                       id, NULL,
                                       runtime_ref,
                                       app_id_dir, app_context, NULL,
+                                      sockets,
                                       FALSE, TRUE, TRUE,
                                       &app_info_path, -1,
                                       &instance_id_host_dir, NULL,
@@ -605,10 +575,14 @@ flatpak_builtin_build (int argc, char **argv, GCancellable *cancellable, GError
     return FALSE;
 
   if (!flatpak_run_add_environment_args (bwrap, app_info_path, run_flags, id,
-                                         app_context, app_id_dir, NULL, -1,
+                                         app_context,
+                                         shares, devices, sockets, features,
+                                         app_id_dir, NULL, -1,
                                          instance_id, NULL, cancellable, error))
     return FALSE;
 
+  add_empty_font_dirs_xml (bwrap);
+
   for (i = 0; opt_bind_mounts != NULL && opt_bind_mounts[i] != NULL; i++)
     {
       char *split = strchr (opt_bind_mounts[i], '=');
@@ -625,9 +599,6 @@ flatpak_builtin_build (int argc, char **argv, GCancellable *cancellable, GError
                               NULL);
     }
 
-  /* Add empty font-dirs.xml only if user hasn't already mapped it */
-  add_empty_font_dirs_xml (bwrap);
-
   if (opt_build_dir != NULL)
     {
       flatpak_bwrap_add_args (bwrap,
