@@ -56,6 +56,7 @@ static char *opt_url;
 static char *opt_collection_id = NULL;
 static gboolean opt_from;
 static char **opt_gpg_import;
+static char *opt_signature_lookaside = NULL;
 static char *opt_authenticator_name = NULL;
 static char **opt_authenticator_options = NULL;
 static gboolean opt_authenticator_install = -1;
@@ -81,6 +82,7 @@ static GOptionEntry common_options[] = {
   { "default-branch", 0, 0, G_OPTION_ARG_STRING, &opt_default_branch, N_("Default branch to use for this remote"), N_("BRANCH") },
   { "collection-id", 0, 0, G_OPTION_ARG_STRING, &opt_collection_id, N_("Collection ID"), N_("COLLECTION-ID") },
   { "gpg-import", 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &opt_gpg_import, N_("Import GPG key from FILE (- for stdin)"), N_("FILE") },
+  { "signature-lookaside", 0, 0, G_OPTION_ARG_STRING, &opt_signature_lookaside, N_("Load signatures from URL"), N_("URL") },
   { "filter", 0, 0, G_OPTION_ARG_FILENAME, &opt_filter, N_("Set path to local filter FILE"), N_("FILE") },
   { "disable", 0, 0, G_OPTION_ARG_NONE, &opt_disable, N_("Disable the remote"), NULL },
   { "authenticator-name", 0, 0, G_OPTION_ARG_STRING, &opt_authenticator_name, N_("Name of authenticator"), N_("NAME") },
@@ -197,6 +199,9 @@ get_config_from_opts (GKeyFile *config,
         return FALSE;
     }
 
+  if (opt_signature_lookaside)
+    g_key_file_set_string (config, group, "xa.signature-lookaside", opt_signature_lookaside);
+
   if (opt_authenticator_name)
     {
       g_key_file_set_string (config, group, "xa.authenticator-name", opt_authenticator_name);
