@@ -41,6 +41,7 @@ static gboolean opt_subsets;
 static gchar *opt_metadata_branch;
 static gchar *opt_commits_branch;
 static gchar *opt_subset;
+static gboolean opt_json;
 
 static gboolean
 ostree_repo_mode_to_string (OstreeRepoMode mode,
@@ -377,7 +378,7 @@ print_branches (OstreeRepo *repo,
 
   flatpak_table_printer_sort (printer, (GCompareFunc) strcmp);
 
-  flatpak_table_printer_print (printer);
+  opt_json ? flatpak_table_printer_print_json (printer) : flatpak_table_printer_print (printer);
 }
 
 static void
@@ -426,7 +427,7 @@ print_subsets (OstreeRepo *repo,
         }
     }
 
-  flatpak_table_printer_print (printer);
+  opt_json ? flatpak_table_printer_print_json (printer) : flatpak_table_printer_print (printer);
 }
 
 
@@ -712,6 +713,7 @@ static GOptionEntry options[] = {
   { "commits", 0, 0, G_OPTION_ARG_STRING, &opt_commits_branch, N_("Show commits for a branch"), N_("BRANCH") },
   { "subsets", 0, 0, G_OPTION_ARG_NONE, &opt_subsets, N_("Print information about the repo subsets"), NULL },
   { "subset", 0, 0, G_OPTION_ARG_STRING, &opt_subset, N_("Limit information to subsets with this prefix"), NULL },
+  { "json", 'j', 0, G_OPTION_ARG_NONE, &opt_json, N_("Show output in JSON format"), NULL },
   { NULL }
 };
 
