@@ -317,7 +317,8 @@ progress_changed_cb (FlatpakTransactionProgress *progress,
           guint64 total_time = elapsed_time * 100 / (double) percent;
           remaining = format_duration (total_time - elapsed_time);
         }
-      speed = g_strdup_printf ("%s/s%s%s", formatted_bytes_sec, remaining ? "  " : "", remaining ? remaining : "");
+      /* Formatted size/remaining time in seconds */
+      speed = g_strdup_printf (_("%s/s%s%s"), formatted_bytes_sec, remaining ? "  " : "", remaining ? remaining : "");
       cli->speed_len = MAX (cli->speed_len, strlen (speed) + 2);
     }
 
@@ -353,7 +354,9 @@ progress_changed_cb (FlatpakTransactionProgress *progress,
     g_string_append (str, " ");
 
   g_string_append (str, " ");
-  g_string_append_printf (str, "%3d%%", percent);
+  /* Download progress percentage, use the appropriate
+    percent format for your language */
+  g_string_append_printf (str, _("%3d%%"), percent);
 
   if (speed)
     g_string_append_printf (str, "  %s", speed);
@@ -594,6 +597,18 @@ operation_error (FlatpakTransaction            *transaction,
         }
     }
 
+  /* On a fatal error, just clear the progress line. The error will be printed in main() before exiting. */
+  if (!non_fatal && self->stop_on_first_error)
+    {
+      if (flatpak_fancy_output ())
+        {
+          flatpak_table_printer_set_cell (self->printer, self->progress_row, 0, "");
+          redraw (self);
+        }
+
+      return FALSE;
+    }
+
   if (flatpak_fancy_output ())
     {
       flatpak_table_printer_set_cell (self->printer, self->progress_row, 0, text);
@@ -605,9 +620,6 @@ operation_error (FlatpakTransaction            *transaction,
   else
     g_printerr ("%s\n", text);
 
-  if (!non_fatal && self->stop_on_first_error)
-    return FALSE;
-
   return TRUE; /* Continue */
 }
 
@@ -1350,7 +1362,7 @@ transaction_ready_pre_auth (FlatpakTransaction *transaction)
   GList *l;
   int i;
   FlatpakTablePrinter *printer;
-  const char *op_shorthand[] = { "i", "u", "i", "r" };
+  const char *op_shorthand[] = { "i", "u", "i", "r", "i" };
 
   /* These caches may no longer be valid once the transaction runs */
   g_clear_pointer (&self->runtime_app_map, g_hash_table_unref);
