@@ -190,8 +190,8 @@ install_authenticator (FlatpakTransaction            *old_transaction,
 {
   g_autoptr(FlatpakTransaction)  transaction2 = NULL;
   g_autoptr(GError) local_error = NULL;
-  FlatpakInstallation *installation = flatpak_transaction_get_installation (old_transaction);
-  FlatpakDir *dir = flatpak_installation_get_dir (installation, NULL);
+  g_autoptr(FlatpakInstallation) installation = flatpak_transaction_get_installation (old_transaction);
+  g_autoptr(FlatpakDir) dir = flatpak_installation_get_dir (installation, NULL);
 
   if (dir == NULL)
     {
@@ -246,7 +246,7 @@ end_of_lifed_with_rebase (FlatpakTransaction *transaction,
 
       g_print (_("Updating to rebased version\n"));
 
-      if (!flatpak_transaction_add_rebase (transaction, remote, rebased_to_ref, NULL, previous_ids, &error))
+      if (!flatpak_transaction_add_rebase_and_uninstall (transaction, remote, rebased_to_ref, ref, NULL, previous_ids, &error))
         {
           g_printerr (_("Failed to rebase %s to %s: %s\n"),
                       flatpak_ref_get_name (rref), rebased_to_ref, error->message);
@@ -254,20 +254,6 @@ end_of_lifed_with_rebase (FlatpakTransaction *transaction,
           return FALSE;
         }
 
-      if (!flatpak_transaction_add_uninstall (transaction, ref, &error))
-        {
-          /* NOT_INSTALLED error is expected in case the op that triggered this was install not update */
-          if (g_error_matches (error, FLATPAK_ERROR, FLATPAK_ERROR_NOT_INSTALLED))
-            g_clear_error (&error);
-          else
-            {
-              g_printerr (_("Failed to uninstall %s for rebase to %s: %s\n"),
-                          flatpak_ref_get_name (rref), rebased_to_ref, error->message);
-              self->got_error = TRUE;
-              return FALSE;
-            }
-        }
-
       return TRUE;
     }
 
