@@ -34,6 +34,7 @@
 
 #include <glib/gi18n-lib.h>
 
+#include <glib-unix.h>
 #include <gio/gio.h>
 #include "libglnx.h"
 
@@ -68,6 +69,9 @@ flatpak_bwrap_new (char **env)
   else
     bwrap->envp = g_get_environ ();
 
+  bwrap->sync_fds[0] = -1;
+  bwrap->sync_fds[1] = -1;
+
   return bwrap;
 }
 
@@ -78,6 +82,7 @@ flatpak_bwrap_free (FlatpakBwrap *bwrap)
   g_array_unref (bwrap->noinherit_fds);
   g_array_unref (bwrap->fds);
   g_strfreev (bwrap->envp);
+  g_clear_pointer (&bwrap->runtime_dir_members, g_ptr_array_unref);
   g_free (bwrap);
 }
 
@@ -268,7 +273,7 @@ flatpak_bwrap_add_args_data (FlatpakBwrap *bwrap,
   if (!flatpak_buffer_to_sealed_memfd_or_tmpfile (&args_tmpf, name, content, content_size, error))
     return FALSE;
 
-  flatpak_bwrap_add_args_data_fd (bwrap, "--ro-bind-data", glnx_steal_fd (&args_tmpf.fd), path);
+  flatpak_bwrap_add_args_data_fd (bwrap, "--ro-bind-data", g_steal_fd (&args_tmpf.fd), path);
   return TRUE;
 }
 
@@ -370,9 +375,9 @@ flatpak_bwrap_bundle_args (FlatpakBwrap *bwrap,
   if (!flatpak_buffer_to_sealed_memfd_or_tmpfile (&args_tmpf, "bwrap-args", data, data_len, error))
     return FALSE;
 
-  fd = glnx_steal_fd (&args_tmpf.fd);
+  fd = g_steal_fd (&args_tmpf.fd);
 
-  flatpak_debug2 ("bwrap --args %d = ...", fd);
+  g_debug ("bwrap --args %d = ...", fd);
 
   for (i = start; i < end; i++)
     {
@@ -380,11 +385,11 @@ flatpak_bwrap_bundle_args (FlatpakBwrap *bwrap,
         {
           g_autofree char *quoted = g_shell_quote (bwrap->argv->pdata[i]);
 
-          flatpak_debug2 ("    %s", quoted);
+          g_debug ("    %s", quoted);
         }
       else
         {
-          flatpak_debug2 ("    %s", (const char *) bwrap->argv->pdata[i]);
+          g_debug ("    %s", (const char *) bwrap->argv->pdata[i]);
         }
     }
 
@@ -495,8 +500,14 @@ flatpak_bwrap_child_setup (GArray *fd_array,
 {
   int i;
 
+  /* There is a dead-lock in glib versions before 2.60 when it closes
+   * the fds. See:  https://gitlab.gnome.org/GNOME/glib/merge_requests/490
+   * This was hitting the test-suite a lot, so we work around it by using
+   * the G_SPAWN_LEAVE_DESCRIPTORS_OPEN/G_SUBPROCESS_FLAGS_INHERIT_FDS flag
+   * and setting CLOEXEC ourselves.
+   */
   if (close_fd_workaround)
-    flatpak_close_fds_workaround (3);
+    g_fdwalk_set_cloexec (3);
 
   /* If no fd_array was specified, don't care. */
   if (fd_array == NULL)
@@ -526,3 +537,29 @@ flatpak_bwrap_child_setup_cb (gpointer user_data)
 
   flatpak_bwrap_child_setup (fd_array, TRUE);
 }
+
+/* Unset FD_CLOEXEC on the array of fds passed in @user_data,
+ * but do not set FD_CLOEXEC on all other fds */
+void
+flatpak_bwrap_child_setup_inherit_fds_cb (gpointer user_data)
+{
+  GArray *fd_array = user_data;
+
+  flatpak_bwrap_child_setup (fd_array, FALSE);
+}
+
+/* Add a --sync-fd argument for bwrap(1). Returns the write end of the pipe on
+ * success, or -1 on error. */
+int
+flatpak_bwrap_add_sync_fd (FlatpakBwrap *bwrap)
+{
+  /* --sync-fd is only allowed once */
+  if (bwrap->sync_fds[1] >= 0)
+    return bwrap->sync_fds[1];
+
+  if (pipe2 (bwrap->sync_fds, O_CLOEXEC) < 0)
+    return -1;
+
+  flatpak_bwrap_add_args_data_fd (bwrap, "--sync-fd", bwrap->sync_fds[0], NULL);
+  return bwrap->sync_fds[1];
+}
