@@ -60,6 +60,8 @@ GType flatpak_deploy_get_type (void);
 
 #define FLATPAK_CLI_UPDATE_INTERVAL_MS 300
 
+typedef struct _PolkitSubject PolkitSubject;
+
 typedef struct
 {
   FlatpakDecomposed *ref;
@@ -87,6 +89,7 @@ typedef struct
 {
   char     *remote_name;
   gboolean  is_file_uri;
+  gboolean  is_oci;
   char     *collection_id;
 
   /* New format summary */
@@ -107,6 +110,7 @@ typedef struct
   int       refcount;
   gint32    default_token_type;
   GPtrArray *sideload_repos;
+  GPtrArray *sideload_image_collections;
 } FlatpakRemoteState;
 
 FlatpakRemoteState *flatpak_remote_state_ref (FlatpakRemoteState *remote_state);
@@ -126,17 +130,20 @@ gboolean flatpak_remote_state_ensure_subsummary_all_arches (FlatpakRemoteState *
                                                             GError            **error);
 gboolean flatpak_remote_state_allow_ref (FlatpakRemoteState *self,
                                          const char *ref);
-gboolean flatpak_remote_state_lookup_ref (FlatpakRemoteState *self,
-                                          const char         *ref,
-                                          char              **out_checksum,
-                                          guint64            *out_timestamp,
-                                          VarRefInfoRef      *out_info,
-                                          GFile             **out_sideload_path,
-                                          GError            **error);
+gboolean flatpak_remote_state_lookup_ref (FlatpakRemoteState  *self,
+                                          const char          *ref,
+                                          char               **out_checksum,
+                                          guint64             *out_timestamp,
+                                          GVariant           **out_summary_metadata,
+                                          GFile              **out_sideload_path,
+                                          FlatpakImageSource **out_image_Source,
+                                          GError             **error);
 GPtrArray *flatpak_remote_state_match_subrefs (FlatpakRemoteState *self,
                                                FlatpakDecomposed *ref);
-GFile *flatpak_remote_state_lookup_sideload_checksum (FlatpakRemoteState *self,
-                                                      char               *checksum);
+void flatpak_remote_state_lookup_sideload_checksum (FlatpakRemoteState  *self,
+                                                    char                *checksum,
+                                                    GFile              **out_sideload_path,
+                                                    FlatpakImageSource **out_image_source);
 gboolean flatpak_remote_state_lookup_cache (FlatpakRemoteState *self,
                                             const char         *ref,
                                             guint64            *download_size,
@@ -163,6 +170,15 @@ GVariant *flatpak_remote_state_load_ref_commit (FlatpakRemoteState *self,
                                                 GError            **error);
 void flatpak_remote_state_add_sideload_dir (FlatpakRemoteState *self,
                                             GFile              *path);
+void flatpak_remote_state_add_sideload_image_collection (FlatpakRemoteState     *self,
+                                                         FlatpakImageCollection *image_collection);
+FlatpakImageSource * flatpak_remote_state_fetch_image_source (FlatpakRemoteState  *self,
+                                                              FlatpakDir          *dir,
+                                                              const char          *ref,
+                                                              const char          *opt_rev,
+                                                              const char          *token,
+                                                              GCancellable        *cancellable,
+                                                              GError             **error);
 
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (FlatpakDir, g_object_unref)
@@ -180,6 +196,7 @@ typedef enum {
   FLATPAK_HELPER_DEPLOY_FLAGS_APP_HINT = 1 << 5,
   FLATPAK_HELPER_DEPLOY_FLAGS_INSTALL_HINT = 1 << 6,
   FLATPAK_HELPER_DEPLOY_FLAGS_UPDATE_PINNED = 1 << 7,
+  FLATPAK_HELPER_DEPLOY_FLAGS_UPDATE_PREINSTALLED = 1 << 8,
 } FlatpakHelperDeployFlags;
 
 #define FLATPAK_HELPER_DEPLOY_FLAGS_ALL (FLATPAK_HELPER_DEPLOY_FLAGS_UPDATE | \
@@ -189,18 +206,21 @@ typedef enum {
                                          FLATPAK_HELPER_DEPLOY_FLAGS_NO_INTERACTION | \
                                          FLATPAK_HELPER_DEPLOY_FLAGS_APP_HINT | \
                                          FLATPAK_HELPER_DEPLOY_FLAGS_INSTALL_HINT | \
-                                         FLATPAK_HELPER_DEPLOY_FLAGS_UPDATE_PINNED)
+                                         FLATPAK_HELPER_DEPLOY_FLAGS_UPDATE_PINNED | \
+                                         FLATPAK_HELPER_DEPLOY_FLAGS_UPDATE_PREINSTALLED)
 
 typedef enum {
   FLATPAK_HELPER_UNINSTALL_FLAGS_NONE = 0,
   FLATPAK_HELPER_UNINSTALL_FLAGS_KEEP_REF = 1 << 0,
   FLATPAK_HELPER_UNINSTALL_FLAGS_FORCE_REMOVE = 1 << 1,
   FLATPAK_HELPER_UNINSTALL_FLAGS_NO_INTERACTION = 1 << 2,
+  FLATPAK_HELPER_UNINSTALL_FLAGS_UPDATE_PREINSTALLED = 1 << 3,
 } FlatpakHelperUninstallFlags;
 
 #define FLATPAK_HELPER_UNINSTALL_FLAGS_ALL (FLATPAK_HELPER_UNINSTALL_FLAGS_KEEP_REF | \
                                             FLATPAK_HELPER_UNINSTALL_FLAGS_FORCE_REMOVE | \
-                                            FLATPAK_HELPER_UNINSTALL_FLAGS_NO_INTERACTION)
+                                            FLATPAK_HELPER_UNINSTALL_FLAGS_NO_INTERACTION | \
+                                            FLATPAK_HELPER_UNINSTALL_FLAGS_UPDATE_PREINSTALLED)
 
 typedef enum {
   FLATPAK_HELPER_CONFIGURE_REMOTE_FLAGS_NONE = 0,
@@ -366,6 +386,19 @@ gboolean        flatpak_remove_override_keyfile (const char  *app_id,
                                                  gboolean     user,
                                                  GError     **error);
 
+typedef struct
+{
+  FlatpakDecomposed *ref;
+  char *collection_id;
+} FlatpakPreinstallConfig;
+
+GPtrArray * flatpak_get_preinstall_config (const char    *default_arch,
+                                           GCancellable  *cancellable,
+                                           GError       **error);
+gboolean flatpak_dir_uninitialized_mark_preinstalled (FlatpakDir       *self,
+                                                      const GPtrArray  *preinstall_config,
+                                                      GError          **error);
+
 int          flatpak_deploy_data_get_version                     (GBytes *deploy_data);
 const char * flatpak_deploy_data_get_origin                      (GBytes *deploy_data);
 const char * flatpak_deploy_data_get_commit                      (GBytes *deploy_data);
@@ -576,6 +609,7 @@ gboolean              flatpak_dir_pull                                      (Fla
                                                                              const char                    *opt_rev,
                                                                              const char                   **subpaths,
                                                                              GFile                         *sideload_repo,
+                                                                             FlatpakImageSource            *opt_image_source,
                                                                              GBytes                        *require_metadata,
                                                                              const char                    *token,
                                                                              OstreeRepo                    *repo,
@@ -682,6 +716,7 @@ gboolean              flatpak_dir_deploy_install                            (Fla
                                                                              const char                   **previous_ids,
                                                                              gboolean                       reinstall,
                                                                              gboolean                       pin_on_deploy,
+                                                                             gboolean                       update_preinstalled_on_deploy,
                                                                              GCancellable                  *cancellable,
                                                                              GError                       **error);
 gboolean              flatpak_dir_install                                   (FlatpakDir                    *self,
@@ -691,12 +726,14 @@ gboolean              flatpak_dir_install                                   (Fla
                                                                              gboolean                       reinstall,
                                                                              gboolean                       app_hint,
                                                                              gboolean                       pin_on_deploy,
+                                                                             gboolean                       update_preinstalled_on_deploy,
                                                                              FlatpakRemoteState            *state,
                                                                              FlatpakDecomposed             *ref,
                                                                              const char                    *opt_commit,
                                                                              const char                   **subpaths,
                                                                              const char                   **previous_ids,
                                                                              GFile                         *sideload_repo,
+                                                                             FlatpakImageSource            *opt_image_source,
                                                                              GBytes                        *require_metadata,
                                                                              const char                    *token,
                                                                              FlatpakProgress               *progress,
@@ -744,6 +781,7 @@ gboolean              flatpak_dir_update                                    (Fla
                                                                              const char                   **opt_subpaths,
                                                                              const char                   **opt_previous_ids,
                                                                              GFile                         *sideload_repo,
+                                                                             FlatpakImageSource            *opt_image_source,
                                                                              GBytes                        *require_metadata,
                                                                              const char                    *token,
                                                                              FlatpakProgress               *progress,
@@ -983,6 +1021,7 @@ gboolean              flatpak_dir_find_latest_rev                           (Fla
                                                                              char                         **out_rev,
                                                                              guint64                       *out_timestamp,
                                                                              GFile                        **out_sideload_path,
+                                                                             FlatpakImageSource           **out_image_source,
                                                                              GCancellable                  *cancellable,
                                                                              GError                       **error);
 FlatpakDecomposed *   flatpak_dir_get_remote_auto_install_authenticator_ref (FlatpakDir                    *self,
@@ -992,9 +1031,8 @@ char **               flatpak_dir_get_default_locale_languages              (Fla
 char **               flatpak_dir_get_locales                               (FlatpakDir                    *self);
 char **               flatpak_dir_get_locale_languages                      (FlatpakDir                    *self);
 char **               flatpak_dir_get_locale_subpaths                       (FlatpakDir                    *self);
-void                  flatpak_dir_set_source_pid                            (FlatpakDir                    *self,
-                                                                             pid_t                          pid);
-pid_t                 flatpak_dir_get_source_pid                            (FlatpakDir                    *self);
+void                  flatpak_dir_set_subject                               (FlatpakDir                    *self,
+                                                                             PolkitSubject                 *subject);
 gboolean              flatpak_dir_delete_mirror_refs                        (FlatpakDir                    *self,
                                                                              gboolean                       dry_run,
                                                                              GCancellable                  *cancellable,
@@ -1007,5 +1045,10 @@ char **               flatpak_dir_list_unused_refs                          (Fla
                                                                              FlatpakDirFilterFlags          filter_flags,
                                                                              GCancellable                  *cancellable,
                                                                              GError                       **error);
+gboolean              flatpak_dir_pull_oci_extra_data                       (OstreeRepo              *repo,
+                                                                             FlatpakImageSource      *image_source,
+                                                                             const char              *rev,
+                                                                             GCancellable            *cancellable,
+                                                                             GError                 **error);
 
 #endif /* __FLATPAK_DIR_H__ */
