@@ -157,6 +157,7 @@ struct _FlatpakExports
   FlatpakFilesystemMode host_etc;
   FlatpakFilesystemMode host_os;
   int                   host_fd;
+  FlatpakFilesystemMode host_root;
   FlatpakExportsTestFlags test_flags;
 };
 
@@ -643,6 +644,20 @@ flatpak_exports_append_bwrap_args (FlatpakExports *exports,
                                 etc_bind_mode, "/etc", "/run/host/etc", NULL);
     }
 
+  g_assert (exports->host_root >= FLATPAK_FILESYSTEM_MODE_NONE);
+  g_assert (exports->host_root <= FLATPAK_FILESYSTEM_MODE_LAST);
+
+  if (exports->host_root != FLATPAK_FILESYSTEM_MODE_NONE)
+    {
+      const char *root_bind_mode = "--bind";
+
+      if (exports->host_root == FLATPAK_FILESYSTEM_MODE_READ_ONLY)
+        root_bind_mode = "--ro-bind";
+
+      flatpak_bwrap_add_args (bwrap,
+                              root_bind_mode, "/", "/run/host/root", NULL);
+    }
+
   /* As per the os-release specification https://www.freedesktop.org/software/systemd/man/os-release.html
    * always read-only bind-mount /etc/os-release if it exists, or /usr/lib/os-release as a fallback from
    * the host into the application's /run/host */
@@ -1134,3 +1149,13 @@ flatpak_exports_add_host_os_expose (FlatpakExports       *exports,
 
   exports->host_os = mode;
 }
+
+void
+flatpak_exports_add_host_root_expose (FlatpakExports       *exports,
+                                      FlatpakFilesystemMode mode)
+{
+  g_return_if_fail (mode > FLATPAK_FILESYSTEM_MODE_NONE);
+  g_return_if_fail (mode <= FLATPAK_FILESYSTEM_MODE_LAST);
+
+  exports->host_root = mode;
+}
