@@ -50,13 +50,19 @@ typedef struct FlatpakOciLayerWriter FlatpakOciLayerWriter;
 
 G_DEFINE_AUTOPTR_CLEANUP_FUNC (FlatpakOciLayerWriter, g_object_unref)
 
-FlatpakOciRegistry  *  flatpak_oci_registry_new (const char           *uri,
-                                                 gboolean for_write,
-                                                 int tmp_dfd,
-                                                 GCancellable         * cancellable,
-                                                 GError              **error);
+
+FlatpakOciRegistry  *  flatpak_oci_registry_new (const char    *uri,
+                                                 gboolean       for_write,
+                                                 int            tmp_dfd,
+                                                 GCancellable  *cancellable,
+                                                 GError       **error);
+FlatpakOciRegistry *   flatpak_oci_registry_new_for_archive (GFile        *archive,
+                                                             GCancellable *cancellable,
+                                                             GError      **error);
 void                   flatpak_oci_registry_set_token (FlatpakOciRegistry *self,
                                                        const char *token);
+void                   flatpak_oci_registry_set_signature_lookaside (FlatpakOciRegistry *self,
+                                                                     const char         *signature_lookaside);
 gboolean               flatpak_oci_registry_is_local (FlatpakOciRegistry *self);
 const char          *  flatpak_oci_registry_get_uri (FlatpakOciRegistry *self);
 FlatpakOciIndex     *  flatpak_oci_registry_load_index (FlatpakOciRegistry *self,
@@ -121,9 +127,16 @@ FlatpakOciImage *      flatpak_oci_registry_load_image_config (FlatpakOciRegistr
                                                                gsize              *out_size,
                                                                GCancellable       *cancellable,
                                                                GError            **error);
-FlatpakOciLayerWriter *flatpak_oci_registry_write_layer (FlatpakOciRegistry *self,
-                                                         GCancellable       *cancellable,
-                                                         GError            **error);
+
+typedef enum {
+  FLATPAK_OCI_WRITE_LAYER_FLAGS_NONE = 0,
+  FLATPAK_OCI_WRITE_LAYER_FLAGS_ZSTD = 1 << 0,
+} FlatpakOciWriteLayerFlags;
+
+FlatpakOciLayerWriter *flatpak_oci_registry_write_layer (FlatpakOciRegistry        *self,
+                                                         FlatpakOciWriteLayerFlags  flags,
+                                                         GCancellable              *cancellable,
+                                                         GError                   **error);
 
 int                     flatpak_oci_registry_apply_delta (FlatpakOciRegistry    *self,
                                                           int                    delta_fd,
@@ -153,16 +166,6 @@ gboolean flatpak_archive_read_open_fd_with_checksum (struct archive *a,
                                                      GChecksum      *checksum,
                                                      GError        **error);
 
-GBytes *flatpak_oci_sign_data (GBytes       *data,
-                               const gchar **okey_ids,
-                               const char   *homedir,
-                               GError      **error);
-
-FlatpakOciSignature *flatpak_oci_verify_signature (OstreeRepo *repo,
-                                                   const char *remote_name,
-                                                   GBytes     *signature,
-                                                   GError    **error);
-
 gboolean flatpak_oci_index_ensure_cached (FlatpakHttpSession  *http_session,
                                           const char          *uri,
                                           GFile               *index,
@@ -190,12 +193,8 @@ typedef void (*FlatpakOciPullProgress) (guint64  total_size,
                                         gpointer data);
 
 char * flatpak_pull_from_oci (OstreeRepo            *repo,
-                              FlatpakOciRegistry    *registry,
-                              const char            *oci_repository,
-                              const char            *digest,
-                              const char            *delta_url,
-                              FlatpakOciManifest    *manifest,
-                              FlatpakOciImage       *image_config,
+                              FlatpakImageSource    *image_source,
+                              FlatpakImageSource    *opt_dst_image_source,
                               const char            *remote,
                               const char            *ref,
                               FlatpakPullFlags       flags,
@@ -205,12 +204,9 @@ char * flatpak_pull_from_oci (OstreeRepo            *repo,
                               GError               **error);
 
 gboolean flatpak_mirror_image_from_oci (FlatpakOciRegistry    *dst_registry,
-                                        FlatpakOciRegistry    *registry,
-                                        const char            *oci_repository,
-                                        const char            *digest,
+                                        FlatpakImageSource    *image_source,
                                         const char            *remote,
                                         const char            *ref,
-                                        const char            *delta_url,
                                         OstreeRepo            *repo,
                                         FlatpakOciPullProgress progress_cb,
                                         gpointer               progress_data,
