@@ -232,11 +232,12 @@ create_proxy_socket (const char *template)
 }
 
 gboolean
-flatpak_run_add_session_dbus_args (FlatpakBwrap   *app_bwrap,
-                                   FlatpakBwrap   *proxy_arg_bwrap,
-                                   FlatpakContext *context,
-                                   FlatpakRunFlags flags,
-                                   const char     *app_id)
+flatpak_run_add_session_dbus_args (FlatpakBwrap          *app_bwrap,
+                                   FlatpakBwrap          *proxy_arg_bwrap,
+                                   FlatpakContextSockets  sockets,
+                                   FlatpakContext        *context,
+                                   FlatpakRunFlags        flags,
+                                   const char            *app_id)
 {
   static const char sandbox_socket_path[] = "/run/flatpak/bus";
   static const char sandbox_dbus_address[] = "unix:path=/run/flatpak/bus";
@@ -244,7 +245,7 @@ flatpak_run_add_session_dbus_args (FlatpakBwrap   *app_bwrap,
   const char *dbus_address = g_getenv ("DBUS_SESSION_BUS_ADDRESS");
   g_autofree char *dbus_session_socket = NULL;
 
-  unrestricted = (context->sockets & FLATPAK_CONTEXT_SOCKET_SESSION_BUS) != 0;
+  unrestricted = (sockets & FLATPAK_CONTEXT_SOCKET_SESSION_BUS) != 0;
 
   if (dbus_address != NULL)
     {
@@ -314,17 +315,18 @@ flatpak_run_add_session_dbus_args (FlatpakBwrap   *app_bwrap,
 }
 
 gboolean
-flatpak_run_add_system_dbus_args (FlatpakBwrap   *app_bwrap,
-                                  FlatpakBwrap   *proxy_arg_bwrap,
-                                  FlatpakContext *context,
-                                  FlatpakRunFlags flags)
+flatpak_run_add_system_dbus_args (FlatpakBwrap          *app_bwrap,
+                                  FlatpakBwrap          *proxy_arg_bwrap,
+                                  FlatpakContextSockets  sockets,
+                                  FlatpakContext        *context,
+                                  FlatpakRunFlags        flags)
 {
   gboolean unrestricted, no_proxy;
   const char *dbus_address = g_getenv ("DBUS_SYSTEM_BUS_ADDRESS");
   g_autofree char *real_dbus_address = NULL;
   g_autofree char *dbus_system_socket = NULL;
 
-  unrestricted = (context->sockets & FLATPAK_CONTEXT_SOCKET_SYSTEM_BUS) != 0;
+  unrestricted = (sockets & FLATPAK_CONTEXT_SOCKET_SYSTEM_BUS) != 0;
   if (unrestricted)
     g_info ("Allowing system-dbus access");
 
@@ -390,6 +392,7 @@ flatpak_run_add_a11y_dbus_args (FlatpakBwrap    *app_bwrap,
   g_autoptr(GDBusMessage) msg = NULL;
   g_autofree char *proxy_socket = NULL;
   gboolean sandboxed;
+  const char *value;
 
   if ((flags & FLATPAK_RUN_FLAG_NO_A11Y_BUS_PROXY) != 0)
     return FALSE;
@@ -398,26 +401,45 @@ flatpak_run_add_a11y_dbus_args (FlatpakBwrap    *app_bwrap,
   if (session_bus == NULL)
     return FALSE;
 
-  msg = g_dbus_message_new_method_call ("org.a11y.Bus", "/org/a11y/bus", "org.a11y.Bus", "GetAddress");
-  g_dbus_message_set_body (msg, g_variant_new ("()"));
-  reply =
-    g_dbus_connection_send_message_with_reply_sync (session_bus, msg,
-                                                    G_DBUS_SEND_MESSAGE_FLAGS_NONE,
-                                                    30000,
-                                                    NULL,
-                                                    NULL,
-                                                    NULL);
-  if (reply)
+  value = g_getenv ("AT_SPI_BUS_ADDRESS");
+
+  if (value != NULL && value[0] != '\0')
     {
-      if (g_dbus_message_to_gerror (reply, &local_error))
+      a11y_address = g_strdup (value);
+      g_debug ("Retrieved AT-SPI bus address \"%s\" from environment",
+               a11y_address);
+    }
+
+  /* To have an accurate emulation of AT-SPI's behaviour, ideally we would
+   * also query the AT_SPI_BUS atom on the root window if a11y_address is
+   * still NULL here, but that would require a libX11 dependency, and
+   * isn't done when running under native Wayland anyway. */
+
+  if (a11y_address == NULL)
+    {
+      msg = g_dbus_message_new_method_call ("org.a11y.Bus", "/org/a11y/bus", "org.a11y.Bus", "GetAddress");
+      g_dbus_message_set_body (msg, g_variant_new ("()"));
+      reply =
+        g_dbus_connection_send_message_with_reply_sync (session_bus, msg,
+                                                        G_DBUS_SEND_MESSAGE_FLAGS_NONE,
+                                                        30000,
+                                                        NULL,
+                                                        NULL,
+                                                        NULL);
+      if (reply)
         {
-          if (!g_error_matches (local_error, G_DBUS_ERROR, G_DBUS_ERROR_SERVICE_UNKNOWN))
-            g_message ("Can't find a11y bus: %s", local_error->message);
-        }
-      else
-        {
-          g_variant_get (g_dbus_message_get_body (reply),
-                         "(s)", &a11y_address);
+          if (g_dbus_message_to_gerror (reply, &local_error))
+            {
+              if (!g_error_matches (local_error, G_DBUS_ERROR, G_DBUS_ERROR_SERVICE_UNKNOWN))
+                g_message ("Can't find a11y bus: %s", local_error->message);
+            }
+          else
+            {
+              g_variant_get (g_dbus_message_get_body (reply),
+                             "(s)", &a11y_address);
+              g_debug ("Retrieved AT-SPI bus address \"%s\" from session bus",
+                       a11y_address);
+            }
         }
     }
 
