@@ -10,9 +10,6 @@ main (int argc, char *argv[])
   const char *url, *dest;
   int flags = 0;
 
-  /* Avoid weird recursive type initialization deadlocks from libsoup */
-  g_type_ensure (G_TYPE_SOCKET);
-
   if (argc == 3)
     {
       url = argv[1];
