@@ -13,6 +13,7 @@ import http.server as http_server
 
 repositories = {}
 icons = {}
+signatures = {}
 
 
 def get_index():
@@ -116,6 +117,15 @@ class RequestHandler(http_server.BaseHTTPRequestHandler):
             response = 200
             response_string = icons[self.matches["filename"]]
             response_content_type = "image/png"
+        elif self.check_route("/sig-lookaside/@ref/@sig"):
+            ref = self.matches["ref"]
+            sig = self.matches["sig"]
+            index = int(sig.removeprefix("signature-")) - 1
+            try:
+                response = 200
+                response_string = signatures[ref][index]
+            except (KeyError, IndexError):
+                response = 404
 
         assert isinstance(response, int)
         assert isinstance(response_string, bytes)
@@ -212,6 +222,20 @@ class RequestHandler(http_server.BaseHTTPRequestHandler):
             self.send_response(200)
             self.end_headers()
             return
+        elif self.check_route("/testing-sig/@repo_name/@digest"):
+            repo_name = self.matches["repo_name"]
+            digest = self.matches["digest"]
+            s = self.query["s"][0]
+
+            with open(s, "rb") as f:
+                signature_bytes = f.read()
+
+            digest = digest.replace(":", "=")
+            ref = f"{repo_name}@{digest}"
+            sigs = signatures.setdefault(ref, [])
+            sigs.append(signature_bytes)
+            self.send_response(200)
+            self.end_headers()
         else:
             self.send_response(404)
             self.end_headers()
@@ -244,6 +268,16 @@ class RequestHandler(http_server.BaseHTTPRequestHandler):
             self.send_response(200)
             self.end_headers()
             return
+        elif self.check_route("/testing-sig/@repo_name/@digest"):
+            repo_name = self.matches["repo_name"]
+            digest = self.matches["digest"]
+
+            digest = digest.replace(":", "=")
+            ref = f"{repo_name}@{digest}"
+            signatures[ref] = list()
+            self.send_response(200)
+            self.end_headers()
+            return
         else:
             self.send_response(404)
             self.end_headers()
