@@ -177,15 +177,10 @@ test_empty_context (void)
   g_assert_cmpuint (g_hash_table_size (context->session_bus_policy), ==, 0);
   g_assert_cmpuint (g_hash_table_size (context->system_bus_policy), ==, 0);
   g_assert_cmpuint (g_hash_table_size (context->generic_policy), ==, 0);
-  g_assert_cmpuint (context->shares, ==, 0);
-  g_assert_cmpuint (context->shares_valid, ==, 0);
-  g_assert_cmpuint (context->sockets, ==, 0);
-  g_assert_cmpuint (context->sockets_valid, ==, 0);
-  g_assert_cmpuint (context->devices, ==, 0);
-  g_assert_cmpuint (context->devices_valid, ==, 0);
-  g_assert_cmpuint (context->features, ==, 0);
-  g_assert_cmpuint (context->features_valid, ==, 0);
-  g_assert_cmpuint (flatpak_context_get_run_flags (context), ==, 0);
+  g_assert_cmpuint (g_hash_table_size (context->shares_permissions), ==, 0);
+  g_assert_cmpuint (g_hash_table_size (context->socket_permissions), ==, 0);
+  g_assert_cmpuint (g_hash_table_size (context->device_permissions), ==, 0);
+  g_assert_cmpuint (g_hash_table_size (context->features_permissions), ==, 0);
 
   exports = flatpak_context_get_exports (context, "com.example.App");
   g_assert_nonnull (exports);
@@ -230,7 +225,7 @@ test_full_context (void)
   g_key_file_set_value (keyfile,
                         FLATPAK_METADATA_GROUP_CONTEXT,
                         FLATPAK_METADATA_KEY_DEVICES,
-                        "dri;all;kvm;shm;");
+                        "dri;all;kvm;shm;if:all:true;if:all:!has-wayland;");
   g_key_file_set_value (keyfile,
                         FLATPAK_METADATA_GROUP_CONTEXT,
                         FLATPAK_METADATA_KEY_FEATURES,
@@ -268,41 +263,33 @@ test_full_context (void)
   flatpak_context_load_metadata (context, keyfile, &error);
   g_assert_no_error (error);
 
-  g_assert_cmpuint (context->shares, ==,
+  FlatpakContextShares shares = flatpak_context_compute_allowed_shares (context, NULL);
+  g_assert_cmpuint (shares, ==,
                     (FLATPAK_CONTEXT_SHARED_NETWORK |
                      FLATPAK_CONTEXT_SHARED_IPC));
-  g_assert_cmpuint (context->shares_valid, ==, context->shares);
-  g_assert_cmpuint (context->devices, ==,
+  FlatpakContextDevices devices = flatpak_context_compute_allowed_devices (context, NULL);
+  g_assert_cmpuint (devices, ==,
                     (FLATPAK_CONTEXT_DEVICE_DRI |
                      FLATPAK_CONTEXT_DEVICE_ALL |
                      FLATPAK_CONTEXT_DEVICE_KVM |
                      FLATPAK_CONTEXT_DEVICE_SHM));
-  g_assert_cmpuint (context->devices_valid, ==, context->devices);
-  g_assert_cmpuint (context->sockets, ==,
-                    (FLATPAK_CONTEXT_SOCKET_X11 |
-                     FLATPAK_CONTEXT_SOCKET_WAYLAND |
+  FlatpakContextSockets sockets = flatpak_context_compute_allowed_sockets (context, NULL);
+  g_assert_cmpuint (sockets, ==,
+                    (FLATPAK_CONTEXT_SOCKET_WAYLAND |
                      FLATPAK_CONTEXT_SOCKET_INHERIT_WAYLAND_SOCKET |
                      FLATPAK_CONTEXT_SOCKET_PULSEAUDIO |
                      FLATPAK_CONTEXT_SOCKET_SESSION_BUS |
                      FLATPAK_CONTEXT_SOCKET_SYSTEM_BUS |
-                     FLATPAK_CONTEXT_SOCKET_FALLBACK_X11 |
                      FLATPAK_CONTEXT_SOCKET_SSH_AUTH |
                      FLATPAK_CONTEXT_SOCKET_PCSC |
                      FLATPAK_CONTEXT_SOCKET_CUPS));
-  g_assert_cmpuint (context->sockets_valid, ==, context->sockets);
-  g_assert_cmpuint (context->features, ==,
+  FlatpakContextFeatures features = flatpak_context_compute_allowed_features (context, NULL);
+  g_assert_cmpuint (features, ==,
                     (FLATPAK_CONTEXT_FEATURE_DEVEL |
                      FLATPAK_CONTEXT_FEATURE_MULTIARCH |
                      FLATPAK_CONTEXT_FEATURE_BLUETOOTH |
                      FLATPAK_CONTEXT_FEATURE_CANBUS |
                      FLATPAK_CONTEXT_FEATURE_PER_APP_DEV_SHM));
-  g_assert_cmpuint (context->features_valid, ==, context->features);
-
-  g_assert_cmpuint (flatpak_context_get_run_flags (context), ==,
-                    (FLATPAK_RUN_FLAG_DEVEL |
-                     FLATPAK_RUN_FLAG_MULTIARCH |
-                     FLATPAK_RUN_FLAG_BLUETOOTH |
-                     FLATPAK_RUN_FLAG_CANBUS));
 
   g_assert_cmpuint (g_hash_table_size (context->env_vars), ==, 3);
   g_assert_true (g_hash_table_contains (context->env_vars, "LD_AUDIT"));
@@ -380,7 +367,6 @@ test_full_context (void)
   g_assert_cmpstr (strv[i++], ==, "ssh-auth");
   g_assert_cmpstr (strv[i++], ==, "system-bus");
   g_assert_cmpstr (strv[i++], ==, "wayland");
-  g_assert_cmpstr (strv[i++], ==, "x11");
   g_assert_cmpstr (strv[i], ==, NULL);
   g_assert_cmpuint (i, ==, n);
   g_clear_pointer (&strv, g_strfreev);
@@ -394,6 +380,8 @@ test_full_context (void)
   i = 0;
   g_assert_cmpstr (strv[i++], ==, "all");
   g_assert_cmpstr (strv[i++], ==, "dri");
+  g_assert_cmpstr (strv[i++], ==, "if:all:!has-wayland");
+  g_assert_cmpstr (strv[i++], ==, "if:all:true");
   g_assert_cmpstr (strv[i++], ==, "kvm");
   g_assert_cmpstr (strv[i++], ==, "shm");
   g_assert_cmpstr (strv[i], ==, NULL);
