@@ -23,7 +23,7 @@ set -euo pipefail
 
 skip_without_bwrap
 
-echo "1..14"
+echo "1..18"
 
 # Start the fake registry server
 
@@ -197,6 +197,21 @@ assert_not_has_file $icondir/64x64
 
 ok "appstream change"
 
+# Build and install an app with zstd-compressed layers
+
+make_updated_app oci "" "" "ZSTD"
+
+${FLATPAK} build-bundle --oci --oci-layer-compress=zstd $FL_GPGARGS repos/oci oci/app-image-zstd org.test.Hello >&2
+file "$(pwd)/oci/app-image-zstd/blobs"/**/* | grep -q Zstandard
+$client add hello latest "$(pwd)/oci/app-image-zstd"
+
+${FLATPAK} ${U} update -y -vv --ostree-verbose org.test.Hello >&2
+
+run org.test.Hello &> hello_out
+assert_file_has_content hello_out '^Hello world, from a sandboxZSTD$'
+
+ok "export and install zstd layers"
+
 # Change the remote to a non-OCI remote, check that we cleaned up
 
 if [ x${USE_SYSTEMDIR-} == xyes ] ; then
@@ -315,3 +330,97 @@ assert_file_has_content remotes-list "^hello-origin.*[ 	]${scheme}://127\.0\.0\.
 assert_not_has_file $base/oci/hello-origin.index.gz
 
 ok "change remote origin via bundle"
+
+${FLATPAK} ${U} -y uninstall org.test.Hello >&2
+${FLATPAK} ${U} -y uninstall org.test.Platform >&2
+
+${FLATPAK} ${U} list --columns=application,origin > flatpak-list
+assert_not_file_has_content flatpak-list 'org.test.Platform'
+assert_not_file_has_content flatpak-list 'org.test.Hello'
+
+${FLATPAK} ${U} remotes --show-disabled > remotes-list
+assert_not_file_has_content remotes-list '^platform-origin'
+assert_not_file_has_content remotes-list '^hello-origin'
+
+ok "clean up"
+
+# Install from registry via a docker:// location
+# TODO: docker:// locations only support HTTPS
+# This needs https://github.com/flatpak/flatpak/pull/5916
+
+if false && [ x${USE_SYSTEMDIR-} != xyes ]; then
+    $FLATPAK --user -y install docker://127.0.0.1/platform-image:latest >&2
+
+    ${FLATPAK} ${U} list --columns=application,origin > flatpak-list
+    assert_file_has_content flatpak-list '^org.test.Platform	*platform-origin$'
+
+    ${FLATPAK} ${U} remotes --show-disabled > remotes-list
+    assert_file_has_content remotes-list '^platform-origin'
+
+    ok "install image from registry"
+else
+    ok "install image from registry # skip  Not supported"
+fi
+
+# Test OCI signing
+
+${FLATPAK} build-bundle --runtime --oci "${FL_GPGARGS}" repos/oci oci/platform-image org.test.Platform >&2
+digest=$(jq -r '.manifests[0].digest' "$(pwd)/oci/platform-image/index.json")
+make_oci_signature "${digest}" "127.0.0.1:${port}/platform:latest" > "$(pwd)/oci/platform-image-signature-1"
+make_oci_signature "${digest}" "127.0.0.1:${port}/platform:latest" "${FL_GPGCMDARGS2}"> "$(pwd)/oci/platform-image-signature-2"
+
+$client add platform latest "$(pwd)/oci/platform-image"
+$client add-signature platform "${digest}" "$(pwd)/oci/platform-image-signature-1"
+
+${FLATPAK} build-bundle --oci "${FL_GPGARGS}" repos/oci oci/app-image org.test.Hello >&2
+digest=$(jq -r '.manifests[0].digest' "$(pwd)/oci/app-image/index.json")
+make_oci_signature "${digest}" "127.0.0.1:${port}/hello:latest" > "$(pwd)/oci/app-image-signature-1"
+make_oci_signature "${digest}" "127.0.0.1:${port}/hello:latest" "$FL_GPGCMDARGS2" > "$(pwd)/oci/app-image-signature-2"
+
+$client add hello latest "$(pwd)/oci/app-image"
+
+${FLATPAK} ${U} remote-add oci-registry-sig "oci+${scheme}://127.0.0.1:${port}" \
+    --signature-lookaside "${scheme}://127.0.0.1:${port}/sig-lookaside" \
+    --gpg-import=${FL_GPG_HOMEDIR}/pubring.gpg >&2
+
+if ${FLATPAK} ${U} install -y oci-registry-sig org.test.Hello >&2; then
+    assert_not_reached "Should fail install due to missing signature key"
+fi
+
+$client add-signature hello "${digest}" "$(pwd)/oci/app-image-signature-2"
+if ${FLATPAK} ${U} install -y oci-registry-sig org.test.Hello >&2; then
+    assert_not_reached "Should fail install due to wrong signature key"
+fi
+
+$client add-signature hello "${digest}" "$(pwd)/oci/app-image-signature-1"
+${FLATPAK} ${U} install -y oci-registry-sig org.test.Hello >&2
+
+${FLATPAK} build-bundle --oci "${FL_GPGARGS}" repos/oci oci/app-image org.test.Hello >&2
+digest=$(jq -r '.manifests[0].digest' "$(pwd)/oci/app-image/index.json")
+make_oci_signature "${digest}" "127.0.0.1:${port}/hello:latest" > "$(pwd)/oci/app-image-signature-1"
+make_oci_signature "${digest}" "127.0.0.1:${port}/hello:latest" "$FL_GPGCMDARGS2" > "$(pwd)/oci/app-image-signature-2"
+
+$client add hello latest "$(pwd)/oci/app-image"
+if ${FLATPAK} update -y org.test.Hello >&2; then
+    assert_not_reached "Should fail install due to outdated signature key"
+fi
+
+$client add-signature hello "${digest}" "$(pwd)/oci/app-image-signature-1"
+${FLATPAK} update -y org.test.Hello >&2
+
+${FLATPAK} uninstall -y org.test.Hello >&2
+
+${FLATPAK} ${U} remote-delete --force oci-registry-sig >&2
+${FLATPAK} ${U} remote-add oci-registry-sig "oci+${scheme}://127.0.0.1:${port}" \
+    --signature-lookaside "${scheme}://127.0.0.1:${port}/sig-lookaside" \
+    --gpg-import=${FL_GPG_HOMEDIR2}/pubring.gpg >&2
+
+if ${FLATPAK} ${U} install -y oci-registry-sig org.test.Hello >&2; then
+    assert_not_reached "Should fail install due to locally changed trusted key"
+fi
+
+$client add-signature hello "${digest}" "$(pwd)/oci/app-image-signature-2"
+
+${FLATPAK} ${U} install -y oci-registry-sig org.test.Hello >&2
+
+ok "signed images"
\ No newline at end of file
