@@ -17,7 +17,7 @@ reset_overrides () {
     assert_file_empty info
 }
 
-echo "1..18"
+echo "1..20"
 
 setup_repo
 install_repo
@@ -29,12 +29,22 @@ ${FLATPAK} override --user --nosocket=ssh-auth org.test.Hello
 ${FLATPAK} override --user --show org.test.Hello > override
 
 assert_file_has_content override "^\[Context\]$"
-assert_file_has_content override "^sockets=wayland;!ssh-auth;$"
+assert_file_has_content override "^sockets=!ssh-auth;wayland;$"
 
 ok "override --socket"
 
 reset_overrides
 
+${FLATPAK} override --user --socket-if=wayland:!has-wayland org.test.Hello
+${FLATPAK} override --user --socket-if=wayland:true org.test.Hello
+${FLATPAK} override --user --show org.test.Hello > override
+
+assert_file_has_content override "^\[Context\]$"
+assert_file_has_content override "^sockets=wayland;if:wayland:!has-wayland;if:wayland:true;$"
+ok "override --socket-if"
+
+reset_overrides
+
 ${FLATPAK} override --user --device=dri org.test.Hello
 ${FLATPAK} override --user --nodevice=kvm org.test.Hello
 ${FLATPAK} override --user --show org.test.Hello > override
@@ -51,18 +61,31 @@ ${FLATPAK} override --user --unshare=ipc org.test.Hello
 ${FLATPAK} override --user --show org.test.Hello > override
 
 assert_file_has_content override "^\[Context\]$"
-assert_file_has_content override "^shared=network;!ipc;$"
+assert_file_has_content override "^shared=!ipc;network;$"
 
 ok "override --share"
 
 reset_overrides
 
+${FLATPAK} override --user --device-if=dri:a org.test.Hello
+${FLATPAK} override --user --device-if=kvm:x --device-if=dri:b org.test.Hello
+${FLATPAK} override --user --device-if=dri:!c --device-if=dri:!d org.test.Hello
+${FLATPAK} override --user --show org.test.Hello > override
+
+assert_file_has_content override "^\[Context\]$"
+assert_file_has_content override "^devices=dri;if:dri:!c;if:dri:!d;if:dri:a;if:dri:b;kvm;if:kvm:x;$"
+ok "override --device-if"
+
+reset_overrides
+
+reset_overrides
+
 ${FLATPAK} override --user --allow=multiarch org.test.Hello
 ${FLATPAK} override --user --disallow=bluetooth org.test.Hello
 ${FLATPAK} override --user --show org.test.Hello > override
 
 assert_file_has_content override "^\[Context\]$"
-assert_file_has_content override "^features=multiarch;!bluetooth;$"
+assert_file_has_content override "^features=!bluetooth;multiarch;$"
 
 ok "override --allow"
 
