@@ -33,13 +33,13 @@ echo "1..9"
 setup_repo
 
 # Ensure we have the full locale extension:
-${FLATPAK} ${U} config  --set languages "*"
+${FLATPAK} ${U} config  --set languages "*" >&2
 
-${FLATPAK} ${U} install -y test-repo org.test.Hello
+${FLATPAK} ${U} install -y test-repo org.test.Hello >&2
 
 mkdir usb_dir
 
-${FLATPAK} ${U} create-usb --destination-repo=repo usb_dir org.test.Hello
+${FLATPAK} ${U} create-usb --destination-repo=repo usb_dir org.test.Hello >&2
 
 assert_has_file usb_dir/repo/config
 assert_has_file usb_dir/repo/summary
@@ -48,11 +48,11 @@ assert_has_file usb_dir/repo/refs/mirrors/org.test.Collection.test/runtime/org.t
 assert_has_file usb_dir/repo/refs/mirrors/org.test.Collection.test/runtime/org.test.Platform/${ARCH}/master
 assert_has_file usb_dir/repo/refs/mirrors/org.test.Collection.test/appstream2/${ARCH}
 
-${FLATPAK} ${U} uninstall -y --all
+${FLATPAK} ${U} uninstall -y --all >&2
 
 ok "created sideloaded repo"
 
-${FLATPAK} ${U} remote-modify --url="http://no.127.0.0.1:${port}/test" test-repo
+${FLATPAK} ${U} remote-modify --url="http://no.127.0.0.1:${port}/test" test-repo >&2
 
 if ${FLATPAK} ${U} install -y test-repo org.test.Hello &> /dev/null; then
     assert_not_reached "Should not be able to install with wrong url"
@@ -68,7 +68,7 @@ SIDELOAD_REPO=$(realpath usb_dir/repo)
 # This one uses the sideload repo in /run, the rest in the flatpak dir
 ln -s $SIDELOAD_REPO $SIDELOAD_REPO_LINK2
 
-${FLATPAK} ${U} install -y test-repo org.test.Hello
+${FLATPAK} ${U} install -y test-repo org.test.Hello >&2
 assert_has_file $FL_DIR/app/org.test.Hello/$ARCH/master/active/metadata
 assert_has_file $FL_DIR/repo/refs/remotes/test-repo/app/org.test.Hello/${ARCH}/master
 assert_not_has_file $FL_DIR/repo/refs/mirrors/org.test.Collection.test/app/org.test.Hello/${ARCH}/master
@@ -77,8 +77,8 @@ ok "installed sideloaded app"
 
 rm $SIDELOAD_REPO_LINK2
 
-${FLATPAK} ${U} uninstall -y org.test.Hello
-${FLATPAK} ${U} install --sideload-repo=${SIDELOAD_REPO} -y test-repo org.test.Hello
+${FLATPAK} ${U} uninstall -y org.test.Hello >&2
+${FLATPAK} ${U} install --sideload-repo=${SIDELOAD_REPO} -y test-repo org.test.Hello >&2
 
 assert_has_file $FL_DIR/app/org.test.Hello/$ARCH/master/active/metadata
 assert_has_file $FL_DIR/repo/refs/remotes/test-repo/app/org.test.Hello/${ARCH}/master
@@ -91,16 +91,16 @@ ok "installed w/ runtime sideload option"
 # Remove old appstream checkouts so we can update from the sideload
 rm -rf $FL_DIR/appstream/test-repo/$ARCH/
 rm -rf $FL_DIR/repo/refs/remotes/test-repo/appstream2/$ARCH
-ostree prune --repo=$FL_DIR/repo --refs-only
+ostree prune --repo=$FL_DIR/repo --refs-only >&2
 
-${FLATPAK} ${U} update --appstream test-repo
+${FLATPAK} ${U} update --appstream test-repo >&2
 
 assert_has_file $FL_DIR/appstream/test-repo/$ARCH/active/appstream.xml
 
 ok "updated sideloaded appstream"
 
-${FLATPAK} ${U} remote-modify --url="http://127.0.0.1:${port}/test" test-repo
-${FLATPAK} ${U} uninstall -y --all
+${FLATPAK} ${U} remote-modify --url="http://127.0.0.1:${port}/test" test-repo >&2
+${FLATPAK} ${U} uninstall -y --all >&2
 
 # Disable sideload repo and "break" online repo
 rm $SIDELOAD_REPO_LINK
@@ -117,7 +117,7 @@ assert_file_has_content httpd-log "GET .*commit .*404"
 
 ln -s $SIDELOAD_REPO $SIDELOAD_REPO_LINK
 
-${FLATPAK} ${U} install -y test-repo org.test.Hello
+${FLATPAK} ${U} install -y test-repo org.test.Hello >&2
 assert_has_file $FL_DIR/app/org.test.Hello/$ARCH/master/active/metadata
 
 # Re-enable online repo
@@ -136,27 +136,27 @@ NEW_COMMIT=$(cat repos/test/refs/heads/app/org.test.Hello/$ARCH/master)
 # (Re)install the new version (online), ensure we used sideload repo for objects shared between new and old version
 SHARED_OBJECT=$(ostree ls --repo=repos/test -C app/org.test.Hello/$ARCH/master /files/share/applications/org.test.Hello.desktop | awk '{ print $5 }')
 SHARED_OBJECT_PATH=$(commit_to_path $SHARED_OBJECT filez)
-${FLATPAK} ${U} uninstall -y app/org.test.Hello/$ARCH/master
+${FLATPAK} ${U} uninstall -y app/org.test.Hello/$ARCH/master >&2
 
 httpd_clear_log
-${FLATPAK} ${U} install -y app/org.test.Hello/$ARCH/master
+${FLATPAK} ${U} install -y app/org.test.Hello/$ARCH/master >&2
 
 assert_not_file_has_content httpd-log "GET /test/${SHARED_OBJECT_PATH}"
 
 # Prepare sideload repo for NEW_COMMIT
-${FLATPAK} ${U} create-usb --destination-repo=repo2 usb_dir org.test.Hello
+${FLATPAK} ${U} create-usb --destination-repo=repo2 usb_dir org.test.Hello >&2
 
 UPDATED_COMMIT=$( ${FLATPAK} ${U} info --show-commit app/org.test.Hello/${ARCH}/master )
 assert_streq "$NEW_COMMIT" "$UPDATED_COMMIT"
 
 # Update again should do nothing
-${FLATPAK} ${U} update -y
+${FLATPAK} ${U} update -y >&2
 UPDATED_COMMIT=$( ${FLATPAK} ${U} info --show-commit app/org.test.Hello/${ARCH}/master )
 assert_streq "$NEW_COMMIT" "$UPDATED_COMMIT"
 
 # Ensure that offline update don't downgrade to older version
-${FLATPAK} ${U} remote-modify --url="http://no.127.0.0.1:${port}/test" test-repo
-${FLATPAK} ${U} update -y
+${FLATPAK} ${U} remote-modify --url="http://no.127.0.0.1:${port}/test" test-repo >&2
+${FLATPAK} ${U} update -y >&2
 UPDATED_COMMIT=$( ${FLATPAK} ${U} info --show-commit app/org.test.Hello/${ARCH}/master )
 assert_streq "$NEW_COMMIT" "$UPDATED_COMMIT"
 
@@ -164,11 +164,11 @@ ok "updates from sideload don't go backwards"
 
 if [ x${USE_SYSTEMDIR-} == xyes ]; then
     # --commit + --system works only as root, so lets just "fake it" by installing the current sideload version
-    ${FLATPAK} ${U} uninstall -y --no-related org.test.Hello
-    ${FLATPAK} ${U} install -y org.test.Hello
+    ${FLATPAK} ${U} uninstall -y --no-related org.test.Hello >&2
+    ${FLATPAK} ${U} install -y org.test.Hello >&2
 else
     # Try (offline) to update to the old version
-    ${FLATPAK} ${U} update -y --commit $OLD_COMMIT org.test.Hello
+    ${FLATPAK} ${U} update -y --commit $OLD_COMMIT org.test.Hello >&2
 fi
 
 UPDATED_COMMIT=$( ${FLATPAK} ${U} info --show-commit app/org.test.Hello/${ARCH}/master )
@@ -184,25 +184,25 @@ mv usb_dir/repo2 usb_dir/repo
 rm $SIDELOAD_REPO_LINK
 
 # And update to it (offline)
-${FLATPAK} ${U} update -y --sideload-repo=${SIDELOAD_REPO}
+${FLATPAK} ${U} update -y --sideload-repo=${SIDELOAD_REPO} >&2
 
 UPDATED_COMMIT=$( ${FLATPAK} ${U} info --show-commit app/org.test.Hello/${ARCH}/master )
 assert_streq "$NEW_COMMIT" "$UPDATED_COMMIT"
 
 # Re enable (go online)
-${FLATPAK} ${U} remote-modify --url="http://127.0.0.1:${port}/test" test-repo
+${FLATPAK} ${U} remote-modify --url="http://127.0.0.1:${port}/test" test-repo >&2
 
 ok "update offline to new version"
 
 assert_file_has_content ${FL_DIR}/repo/config '^gpg-verify-summary=true$'
 assert_not_file_has_content ${FL_DIR}/repo/config '^gpg-verify-summary=false$'
 
-ostree --repo=$FL_DIR/repo config set --group='remote "test-repo"' gpg-verify-summary false
+ostree --repo=$FL_DIR/repo config set --group='remote "test-repo"' gpg-verify-summary false >&2
 
 assert_file_has_content ${FL_DIR}/repo/config '^gpg-verify-summary=false$'
 assert_not_file_has_content ${FL_DIR}/repo/config '^gpg-verify-summary=true$'
 
-${FLATPAK} ${U} update -y
+${FLATPAK} ${U} update -y >&2
 
 assert_file_has_content ${FL_DIR}/repo/config '^gpg-verify-summary=true$'
 assert_not_file_has_content ${FL_DIR}/repo/config '^gpg-verify-summary=false$'
